=== modified file '.bzrignore'
--- .bzrignore	2009-01-03 19:32:48 +0000
+++ .bzrignore	2009-05-14 20:46:25 +0000
@@ -38,3 +38,15 @@
 lib/canonical/launchpad/icing/build/*
 trace.log
 test-appserver-layer-trace.log
+bin
+develop-eggs
+.installed.cfg
+parts
+*.egg-info
+build
+*.egg
+dist
+./eggs
+./download-cache
+./_pythonpath.py
+

=== modified file 'Makefile'
--- Makefile	2009-05-15 03:50:21 +0000
+++ Makefile	2009-05-31 21:44:30 +0000
@@ -3,14 +3,15 @@
 
 PYTHON_VERSION=2.4
 PYTHON=python${PYTHON_VERSION}
-IPYTHON=$(PYTHON) $(shell which ipython)
-PYTHONPATH:=$(shell pwd)/lib:$(shell pwd)/lib/mailman:${PYTHONPATH}
+WD:=$(shell pwd)
+PY=$(WD)/bin/py
+PYTHONPATH:=$(WD)/lib:$(WD)/lib/mailman:${PYTHONPATH}
 VERBOSITY=-vv
 
 TESTFLAGS=-p $(VERBOSITY)
 TESTOPTS=
 
-SHHH=${PYTHON} utilities/shhh.py
+SHHH=${PY} utilities/shhh.py
 STARTSCRIPT=runlaunchpad.py
 HERE:=$(shell pwd)
 
@@ -23,16 +24,9 @@
 
 BZR_VERSION_INFO = bzr-version-info.py
 
-XSLTPROC=xsltproc
 WADL_FILE = lib/canonical/launchpad/apidoc/wadl-$(LPCONFIG).xml
-WADL_XSL = lib/launchpadlib/wadl-to-refhtml.xsl
 API_INDEX = lib/canonical/launchpad/apidoc/index.html
 
-APPSERVER_ENV = \
-  LPCONFIG=${LPCONFIG} \
-  PYTHONPATH=$(PYTHONPATH) \
-  STORM_CEXTENSIONS=1
-
 EXTRA_JS_FILES=lib/canonical/launchpad/icing/MochiKit.js \
 				$(shell $(HERE)/utilities/yui-deps.py) \
 				lib/canonical/launchpad/icing/lazr/build/lazr.js
@@ -47,14 +41,14 @@
 newsampledata:
 	$(MAKE) -C database/schema newsampledata
 
-hosted_branches:
-	$(PYTHON) ./utilities/make-dummy-hosted-branches
+hosted_branches: $(PY)
+	$(PY) ./utilities/make-dummy-hosted-branches
 
 $(WADL_FILE): $(BZR_VERSION_INFO)
-	LPCONFIG=$(LPCONFIG) $(PYTHON) ./utilities/create-lp-wadl.py > $@
+	LPCONFIG=$(LPCONFIG) $(PY) ./utilities/create-lp-wadl.py > $@
 
-$(API_INDEX): $(WADL_FILE) $(WADL_XSL)
-	$(XSLTPROC) $(WADL_XSL) $(WADL_FILE) > $@
+$(API_INDEX): $(WADL_FILE)
+	bin/apiindex $(WADL_FILE) > $@
 
 apidoc: compile $(API_INDEX)
 
@@ -64,13 +58,16 @@
 	make -C sourcecode/loggerhead check PYTHON=${PYTHON} \
 		PYTHON_VERSION=${PYTHON_VERSION} PYTHONPATH=$(PYTHONPATH)
 
-check_merge:
+_pqm_flag:
+	touch _pqm_flag
+
+check_merge: _pqm_flag $(PY)
 	[ `PYTHONPATH= bzr status -S database/schema/ | \
-		grep -v "\(^P\|pending\|security.cfg\|Makefile\|unautovacuumable\)" | wc -l` -eq 0 ]
-	${PYTHON} lib/canonical/tests/test_no_conflict_marker.py
+		grep -v "\(^P\|pending\|security.cfg\|Makefile\|unautovacuumable\|_pythonpath.py\)" | wc -l` -eq 0 ]
+	${PY} lib/canonical/tests/test_no_conflict_marker.py
 
-check_db_merge:
-	${PYTHON} lib/canonical/tests/test_no_conflict_marker.py
+check_db_merge: _pqm_flag $(PY)
+	${PY} lib/canonical/tests/test_no_conflict_marker.py
 
 # This can be removed once we move to zc.buildout and we have versioned
 # dependencies, but for now we run both Launchpad and all other
@@ -81,9 +78,8 @@
 
 check: build
 	# Run all tests. test_on_merge.py takes care of setting up the
-	# database..
-	env PYTHONPATH=$(PYTHONPATH) \
-	${PYTHON} -t ./test_on_merge.py $(VERBOSITY)
+	# database.
+	${PY} -t ./test_on_merge.py $(VERBOSITY)
 
 lint:
 	@bash ./utilities/lint.sh
@@ -91,86 +87,94 @@
 lint-verbose:
 	@bash ./utilities/lint.sh -v
 
-xxxreport:
-	${PYTHON} -t ./utilities/xxxreport.py -f csv -o xxx-report.csv ./
+xxxreport: $(PY)
+	${PY} -t ./utilities/xxxreport.py -f csv -o xxx-report.csv ./
 
-check-configs:
-	${PYTHON} utilities/check-configs.py
+check-configs: $(PY)
+	${PY} utilities/check-configs.py
 
 pagetests: build
-	env PYTHONPATH=$(PYTHONPATH) ${PYTHON} test.py test_pages
+	env PYTHONPATH=$(PYTHONPATH) bin/test test_pages
 
 inplace: build
 
 build: $(BZR_VERSION_INFO) compile apidoc
 
-compile:
+eggs:
+	# Usually this is linked via link-external-dependencies, but in
+	# deployment we create this ourselves.
+	mkdir eggs
+
+# We touch the _pqm_flag file when we want pqm to check out the
+# download cache sources.  Normally, we want a missing download-cache
+# to raise an error, so developers know to do the usual set-up.
+download-cache:
+	@echo "Missing buildout/download-cache."
+	@echo "Developers: please run utilities/link-external-dependencies."
+	[ -e _pqm_flag ]
+	bzr co lp:lp-source-dependencies download-cache
+
+# The download-cache dependency comes *before* eggs so that developers get the
+# warning before the eggs directory is made.  The target for the eggs directory
+# is only there for deployment convenience.
+bin/buildout: download-cache eggs
+	$(PYTHON) bootstrap.py
+
+$(PY): bin/buildout
+	./bin/buildout configuration:instance_name=${LPCONFIG}
+
+compile: $(PY)
 	${SHHH} $(MAKE) -C sourcecode build PYTHON=${PYTHON} \
 	    PYTHON_VERSION=${PYTHON_VERSION} LPCONFIG=${LPCONFIG}
-	${SHHH} LPCONFIG=${LPCONFIG} PYTHONPATH=$(PYTHONPATH) \
-		 $(PYTHON) -t buildmailman.py
+	${SHHH} LPCONFIG=${LPCONFIG} $(PY) -t buildmailman.py
 	${SHHH} sourcecode/lazr-js/tools/build.py \
 		-n launchpad -s lib/canonical/launchpad/javascript \
 		-b lib/canonical/launchpad/icing/build $(EXTRA_JS_FILES)
 
-runners:
-	echo "#!/bin/sh" > bin/runzope;
-	echo "exec $(PYTHON) $(STARTSCRIPT) -C $(CONFFILE)" >> bin/runzope;
-	chmod +x bin/runzope
-	echo "#!/bin/sh" > bin/zopectl;
-	echo "$(PYTHON) $(PWD)/src/zdaemon/zdctl.py \
-	      -S schema.xml \
-	      -C zdaemon.conf -d \$$*" >> bin/zopectl
-	chmod +x bin/zopectl
-
 test_build: build
-	$(PYTHON) test.py $(TESTFLAGS) $(TESTOPTS)
+	bin/test $(TESTFLAGS) $(TESTOPTS)
 
 test_inplace: inplace
-	$(PYTHON) test.py $(TESTFLAGS) $(TESTOPTS)
+	bin/test $(TESTFLAGS) $(TESTOPTS)
 
 ftest_build: build
-	env PYTHONPATH=$(PYTHONPATH) \
-	    $(PYTHON) test.py -f $(TESTFLAGS) $(TESTOPTS)
+	bin/test -f $(TESTFLAGS) $(TESTOPTS)
 
 ftest_inplace: inplace
-	env PYTHONPATH=$(PYTHONPATH) \
-	    $(PYTHON) test.py -f $(TESTFLAGS) $(TESTOPTS)
+	bin/test -f $(TESTFLAGS) $(TESTOPTS)
 
 run: inplace stop
 	$(RM) thread*.request
-	$(APPSERVER_ENV) $(PYTHON) -t $(STARTSCRIPT) \
-		 -r librarian,google-webservice -C $(CONFFILE)
+	bin/run -r librarian,google-webservice -C $(CONFFILE)
 
 start-gdb: inplace stop support_files
 	$(RM) thread*.request
-	$(APPSERVER_ENV) nohup gdb -x run.gdb --args $(PYTHON) -t $(STARTSCRIPT) \
+	nohup gdb -x run.gdb --args bin/run \
 		-r librarian,google-webservice -C $(CONFFILE) \
 		> ${LPCONFIG}-nohup.out 2>&1 &
 
 run_all: inplace stop hosted_branches
 	$(RM) thread*.request
-	$(APPSERVER_ENV) $(PYTHON) -t $(STARTSCRIPT) \
-		 -r librarian,buildsequencer,sftp,mailman,codebrowse,google-webservice \
+	bin/run -r librarian,buildsequencer,sftp,mailman,codebrowse,google-webservice \
 		 -C $(CONFFILE)
 
 run_codebrowse: build
-	BZR_PLUGIN_PATH=bzrplugins PYTHONPATH=lib $(PYTHON) sourcecode/launchpad-loggerhead/start-loggerhead.py -f
+	BZR_PLUGIN_PATH=bzrplugins $(PY) sourcecode/launchpad-loggerhead/start-loggerhead.py -f
 
 start_codebrowse: build
-	BZR_PLUGIN_PATH=$(shell pwd)/bzrplugins PYTHONPATH=lib $(PYTHON) sourcecode/launchpad-loggerhead/start-loggerhead.py
+	BZR_PLUGIN_PATH=$(shell pwd)/bzrplugins $(PY) sourcecode/launchpad-loggerhead/start-loggerhead.py
 
 stop_codebrowse:
-	PYTHONPATH=lib $(PYTHON) sourcecode/launchpad-loggerhead/stop-loggerhead.py
+	$(PY) sourcecode/launchpad-loggerhead/stop-loggerhead.py
 
 pull_branches: support_files
 	# Mirror the hosted branches in the development upload area to the
 	# mirrored area.
-	$(PYTHON) cronscripts/supermirror-pull.py upload
+	$(PY) cronscripts/supermirror-pull.py upload
 
 scan_branches:
 	# Scan branches from the filesystem into the database.
-	$(PYTHON) cronscripts/branch-scanner.py
+	$(PY) cronscripts/branch-scanner.py
 
 sync_branches: pull_branches scan_branches
 
@@ -184,8 +188,7 @@
 # exiting, as running 'make stop' too soon after running 'make start'
 # will not work as expected.
 start: inplace stop support_files
-	$(APPSERVER_ENV) nohup $(PYTHON) -t $(STARTSCRIPT) -C $(CONFFILE) \
-		 > ${LPCONFIG}-nohup.out 2>&1 &
+	nohup bin/run -C $(CONFFILE) > ${LPCONFIG}-nohup.out 2>&1 &
 
 # This is a stripped down version of the "start" target for use on
 # production servers - removes running 'make build' because we already
@@ -194,14 +197,12 @@
 # even if the service is already stopped, and bzr_version_info is not
 # needed either because it's run as part of 'make build'.
 initscript-start:
-	$(APPSERVER_ENV) nohup $(PYTHON) -t $(STARTSCRIPT) -C $(CONFFILE) \
-		 > ${LPCONFIG}-nohup.out 2>&1 &
+	nohup bin/run -C $(CONFFILE) > ${LPCONFIG}-nohup.out 2>&1 &
 
 # Kill launchpad last - other services will probably shutdown with it,
 # so killing them after is a race condition.
 stop: build
-	@ $(APPSERVER_ENV) ${PYTHON} \
-	    utilities/killservice.py librarian buildsequencer launchpad mailman
+	bin/killservice librarian buildsequencer launchpad mailman
 
 shutdown: scheduleoutage stop
 	$(RM) +maintenancetime.txt
@@ -213,19 +214,14 @@
 	sleep ${MINS_TO_SHUTDOWN}m
 
 harness:
-	$(APPSERVER_ENV) $(PYTHON) -i lib/canonical/database/harness.py
+	bin/harness
 
 iharness:
-	$(APPSERVER_ENV) $(IPYTHON) -i lib/canonical/database/harness.py
+	bin/iharness
 
 rebuildfti:
 	@echo Rebuilding FTI indexes on launchpad_dev database
-	$(PYTHON) database/schema/fti.py -d launchpad_dev --force
-
-debug:
-	$(APPSERVER_ENV) \
-		 $(PYTHON) -i -c \ "from zope.app import Application;\
-		    app = Application('Data.fs', 'site.zcml')()"
+	$(PY) database/schema/fti.py -d launchpad_dev --force
 
 clean:
 	$(MAKE) -C sourcecode/pygettextpo clean
@@ -233,6 +229,9 @@
 	    -name '*.o' -o -name '*.so' -o -name '*.la' -o \
 	    -name '*.lo' -o -name '*.py[co]' -o -name '*.dll' \) \
 	    -print0 | xargs -r0 $(RM)
+	$(RM) -r bin
+	$(RM) -r parts
+	$(RM) .installed.cfg
 	$(RM) -r build
 	$(RM) thread*.request
 	$(RM) -r lib/mailman
@@ -240,6 +239,7 @@
 	$(RM) -r $(CODEHOSTING_ROOT)
 	$(RM) $(WADL_FILE) $(API_INDEX)
 	$(RM) $(BZR_VERSION_INFO)
+	$(RM) _pythonpath.py
 	$(RM) -rf \
 			  /var/tmp/builddmaster \
 			  /var/tmp/bzrsync \
@@ -268,8 +268,7 @@
 
 zcmldocs:
 	mkdir -p doc/zcml/namespaces.zope.org
-	PYTHONPATH=$(shell pwd)/src:$(PYTHONPATH) $(PYTHON) \
-	    ./sourcecode/zope/src/zope/configuration/stxdocs.py \
+	bin/stxdocs \
 	    -f sourcecode/zope/src/zope/app/zcmlfiles/meta.zcml \
 	    -o doc/zcml/namespaces.zope.org
 
@@ -277,9 +276,7 @@
 
 # Generate launchpad.pot by extracting message ids from the source
 launchpad.pot:
-	$(PYTHON) sourcecode/zope/utilities/i18nextract.py \
-	    -d launchpad -p lib/canonical/launchpad \
-	    -o locales
+	bin/i18nextract.py
 
 install: reload-apache
 
@@ -300,15 +297,19 @@
 	/etc/init.d/apache2 reload
 
 static:
-	$(PYTHON) scripts/make-static.py
-
-TAGS:
-	ctags -e -R --exclude='*yui/2.6.0*' --exclude='*-min.js' lib/canonical && \
-		ctags -e --exclude=lib/canonical -a -R lib/
-
-tags:
-	ctags -R --exclude='*yui/2.6.0*' --exclude='*-min.js' lib/canonical && \
-		ctags --exclude=lib/canonical -a -R lib/
+	$(PY) scripts/make-static.py
+
+TAGS: compile
+	# emacs tags
+	bin/tags -e
+
+tags: compile
+	# vi tags
+	bin/tags -v
+
+ID: compile
+	# idutils ID file
+	bin/tags -i
 
 .PHONY: apidoc check tags TAGS zcmldocs realclean clean debug stop\
 	start run ftest_build ftest_inplace test_build test_inplace pagetests\

=== added file 'bootstrap.py'
--- bootstrap.py	1970-01-01 00:00:00 +0000
+++ bootstrap.py	2009-05-31 22:17:45 +0000
@@ -0,0 +1,86 @@
+##############################################################################
+#
+# Copyright (c) 2006 Zope Corporation and Contributors.
+# All Rights Reserved.
+#
+# This software is subject to the provisions of the Zope Public License,
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
+# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
+# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
+# FOR A PARTICULAR PURPOSE.
+#
+##############################################################################
+
+# NOTE TO LAUNCHPAD DEVELOPERS: This is a bootstrapping file from the
+# zc.buildout project, which we have hacked slightly.
+# See the docstring below for usage.
+
+"""Bootstrap a buildout-based project
+
+Simply run this script in a directory containing a buildout.cfg.
+The script accepts buildout command-line options, so you can
+use the -c option to specify an alternate configuration file.
+
+$Id$
+"""
+
+import os, shutil, sys, tempfile, urllib2
+
+tmpeggs = tempfile.mkdtemp()
+
+is_jython = sys.platform.startswith('java')
+
+try:
+    import pkg_resources
+except ImportError:
+    ez = {}
+    # exec urllib2.urlopen('http://peak.telecommunity.com/dist/ez_setup.py'
+    #                      ).read() in ez
+    exec open('ez_setup.py').read() in ez
+    ez['use_setuptools'](
+        to_dir=tmpeggs,
+        download_base='file://%s/download-cache/dist/' % (os.getcwd(),),
+        download_delay=0)
+
+    import pkg_resources
+
+if sys.platform == 'win32':
+    def quote(c):
+        if ' ' in c:
+            return '"%s"' % c # work around spawn lamosity on windows
+        else:
+            return c
+else:
+    def quote (c):
+        return c
+
+cmd = 'from setuptools.command.easy_install import main; main()'
+ws  = pkg_resources.working_set
+
+if is_jython:
+    import subprocess
+    
+    assert subprocess.Popen([sys.executable] + ['-c', quote(cmd), '-mqNxd', 
+           quote(tmpeggs), 'zc.buildout'], 
+           env=dict(os.environ,
+               PYTHONPATH=
+               ws.find(pkg_resources.Requirement.parse('setuptools')).location
+               ),
+           ).wait() == 0
+
+else:
+    assert os.spawnle(
+        os.P_WAIT, sys.executable, quote (sys.executable),
+        '-c', quote (cmd), '-mqNxd', quote (tmpeggs), 'zc.buildout',
+        dict(os.environ,
+            PYTHONPATH=
+            ws.find(pkg_resources.Requirement.parse('setuptools')).location
+            ),
+        ) == 0
+
+ws.add_entry(tmpeggs)
+ws.require('zc.buildout')
+import zc.buildout.buildout
+zc.buildout.buildout.main(sys.argv[1:] + ['bootstrap'])
+shutil.rmtree(tmpeggs)

=== added directory 'buildout-templates'
=== renamed file 'cronscripts/_pythonpath.py' => 'buildout-templates/_pythonpath.py.in'
--- cronscripts/_pythonpath.py	2008-06-19 07:57:48 +0000
+++ buildout-templates/_pythonpath.py.in	2009-05-13 22:03:33 +0000
@@ -1,12 +1,12 @@
 # Copyright 2004-2005 Canonical Ltd.  All rights reserved.
 
+# NOTE: This is a generated file.  The original is in
+# buildout-templates/_pythonpath.py.in
+
 __metaclass__ = type
 
-import sys, os, os.path
-
-sys.path.insert(0, os.path.join(
-    os.path.dirname(__file__), os.pardir, 'lib'
-    ))
-
+import sys, os
+
+sys.path[0:0] = [${string-paths}]
 # Enable Storm's C extensions
 os.environ['STORM_CEXTENSIONS'] = '1'

=== added directory 'buildout-templates/bin'
=== added file 'buildout-templates/bin/py.in'
--- buildout-templates/bin/py.in	1970-01-01 00:00:00 +0000
+++ buildout-templates/bin/py.in	2009-05-01 00:25:52 +0000
@@ -0,0 +1,2 @@
+#!/bin/sh
+PYTHONPATH=${os-paths} ${buildout:executable} "$@"

=== renamed file 'test.py' => 'buildout-templates/bin/test.in'
--- test.py	2009-05-06 15:13:39 +0000
+++ buildout-templates/bin/test.in	2009-05-29 16:22:44 +0000
@@ -1,4 +1,4 @@
-#!/usr/bin/python2.4
+#!${buildout:executable}
 ##############################################################################
 #
 # Copyright (c) 2004 Zope Corporation and Contributors.
@@ -13,11 +13,10 @@
 #
 ##############################################################################
 """Test script
-
-$Id: test.py 25177 2004-06-02 13:17:31Z jim $
 """
 import sys, os, time, logging, warnings, re
 
+BUILD_DIR = '${buildout:directory}'
 
 if os.getsid(0) == os.getsid(os.getppid()):
     # We need to become the process group leader so test_on_merge.py
@@ -38,13 +37,7 @@
 # Enable Storm's C extensions
 os.environ['STORM_CEXTENSIONS'] = '1'
 
-here = os.path.dirname(os.path.realpath(__file__))
-sys.path.insert(0, os.path.join(here, 'lib'))
-
-# Mailman lives in a subdirectory.
-mailman_dir = os.path.join(here, 'lib', 'mailman')
-if os.path.isdir(mailman_dir):
-    sys.path.insert(0, mailman_dir)
+sys.path[0:0] = [${string-paths}]
 
 # Set PYTHONPATH environment variable for spawned processes
 os.environ['PYTHONPATH'] = ':'.join(sys.path)
@@ -56,11 +49,11 @@
     main_process = True
 
 # Install the import fascist import hook and atexit handler.
-import importfascist
+from lp.scripts.utilities import importfascist
 importfascist.install_import_fascist()
 
 # Install the warning handler hook and atexit handler.
-import warninghandler
+from lp.scripts.utilities import warninghandler
 warninghandler.install_warning_handler()
 
 # Ensure that atexit handlers are executed on TERM.
@@ -70,8 +63,10 @@
 signal.signal(signal.SIGTERM, exit_with_atexit_handlers)
 
 # Ensure overrides are generated
+sys.path.insert(0, BUILD_DIR)
 from configs import generate_overrides
 generate_overrides()
+del sys.path[0]
 
 # Tell canonical.config to use the testrunner config instance.
 from canonical.config import config
@@ -79,8 +74,7 @@
 
 # Remove this module's directory from path, so that zope.testbrowser
 # can import pystone from test:
-sys.path[:] = [p for p in sys.path if os.path.abspath(p) != here]
-
+sys.path[:] = [p for p in sys.path if os.path.abspath(p) != BUILD_DIR]
 
 # Turn on psycopg debugging wrapper
 #import canonical.database.debug
@@ -95,26 +89,41 @@
 
 # Silence spurious warnings. Note that this does not propagate to subprocesses
 # so this is not always as easy as it seems. Warnings caused by our code that
-# need to be silenced should have an accomponied Bug reference.
+# need to be silenced should have an accompanied Bug reference.
 #
 warnings.filterwarnings(
-        'ignore', 'PyCrypto', RuntimeWarning, 'twisted[.]conch[.]ssh'
-        )
-warnings.filterwarnings(
-        'ignore', 'twisted.python.plugin', DeprecationWarning,
-        )
-warnings.filterwarnings(
-        'ignore', 'bzrlib.*was deprecated', DeprecationWarning
-)
-
+    'ignore', 'PyCrypto', RuntimeWarning, 'twisted[.]conch[.]ssh'
+    )
+warnings.filterwarnings(
+    'ignore', 'twisted.python.plugin', DeprecationWarning,
+    )
+warnings.filterwarnings(
+    'ignore', 'bzrlib.*was deprecated', DeprecationWarning
+)
+# The next one is caused by a lamosity in python-openid.  The following change
+# to openid/server/server.py would make the warning filter unnecessary:
+# 978c974,974
+# >         try:
+# >             namespace = request.message.getOpenIDNamespace()
+# >         except AttributeError:
+# >             namespace = request.namespace
+# >         self.fields = Message(namespace)
+# ---
+# <         self.fields = Message(request.namespace)
+warnings.filterwarnings(
+    'ignore',
+    (r'The \"namespace\" attribute of CheckIDRequest objects is deprecated.\s+'
+     r'Use \"message.getOpenIDNamespace\(\)\" instead'),
+    DeprecationWarning
+)
 # This warning will be triggered if the beforeTraversal hook fails. We
 # want to ensure it is not raised as an error, as this will mask the real
 # problem.
 warnings.filterwarnings(
-        'always',
-        re.escape('clear_request_started() called outside of a request'),
-        UserWarning
-        )
+    'always',
+    re.escape('clear_request_started() called outside of a request'),
+    UserWarning
+    )
 
 # Any warnings not explicitly silenced are errors
 warnings.filterwarnings('error', append=True)
@@ -131,7 +140,7 @@
 defaults = [
     # Find tests in the tests and ftests directories
     '--tests-pattern=^f?tests$',
-    '--test-path=%s' % os.path.join(here, 'lib'),
+    '--test-path=${buildout:directory}/lib',
     '--package=canonical',
     '--package=lp',
     ]
@@ -208,7 +217,7 @@
     # e.g. run the test that you are currently editing.
     try:
         there = os.getcwd()
-        os.chdir(here)
+        os.chdir('${buildout:directory}')
         result = testrunner.run(defaults)
     finally:
         os.chdir(there)

=== added file 'buildout-templates/bin/update-download-cache.in'
--- buildout-templates/bin/update-download-cache.in	1970-01-01 00:00:00 +0000
+++ buildout-templates/bin/update-download-cache.in	2009-05-13 00:29:48 +0000
@@ -0,0 +1,2 @@
+bzr up ${buildout:directory}/buildout/download-cache
+

=== added file 'buildout.cfg'
--- buildout.cfg	1970-01-01 00:00:00 +0000
+++ buildout.cfg	2009-05-29 00:01:54 +0000
@@ -0,0 +1,73 @@
+[buildout]
+parts =
+    scripts
+    filetemplates
+    tags
+    iharness
+    i18n
+unzip = true
+eggs-directory = eggs
+download-cache = download-cache
+
+# Disable this option temporarily if you want buildout to find software
+# dependencies *other* than those in our download-cache.  Once you have the
+# desired software, reenable this option (and check in the new software to
+# lp:lp-source-dependencies if this is going to be reviewed/merged/deployed.)
+install-from-cache = true
+
+# This also will need to be temporarily disabled or changed for package
+# upgrades.  Newly-added packages should also add their desired version number
+# to versions.cfg.
+extends = versions.cfg
+
+# allow-picked-versions = false # A buildout bug is not allowing this to work.
+
+prefer-final = true
+
+develop = .
+
+[configuration]
+instance_name = development
+
+[filetemplates]
+recipe = z3c.recipe.filetemplate
+eggs = lp
+# XXX gary 2009-5-12 bug 375751:
+# Make mailman built and installed in a more normal way.
+extra-paths = ${buildout:directory}/lib/mailman
+source-directory = buildout-templates
+
+[scripts]
+recipe = zc.recipe.egg
+eggs = lp
+# XXX gary 2009-5-12 bug 375751:
+# Make mailman built and installed in a more normal way.
+extra-paths = ${buildout:directory}/lib/mailman
+# note that any indentation is lost in initialization blocks
+initialization = import os
+                 os.environ['STORM_CEXTENSIONS'] = '1'
+                 os.environ['LPCONFIG'] = '${configuration:instance_name}'
+entry-points = stxdocs=zope.configuration.stxdocs:main
+
+[iharness]
+recipe = zc.recipe.egg
+eggs = lp
+       IPython
+extra-paths = ${buildout:directory}/lib/mailman
+initialization = import os
+                 os.environ['STORM_CEXTENSIONS'] = '1'
+                 os.environ['LPCONFIG'] = '${configuration:instance_name}'
+entry-points = iharness=canonical.database.harness:ipython
+scripts = iharness ipython=ipy
+
+[tags]
+recipe = z3c.recipe.tag:tags
+eggs = lp
+
+[i18n]
+recipe = z3c.recipe.i18n:i18n
+eggs = lp
+packages = canonical.launchpad
+domain = launchpad
+output = locales
+zcml = <include file="../../site.zcml" />

=== added symlink 'cronscripts/_pythonpath.py'
=== target is '../_pythonpath.py'
=== added symlink 'database/replication/_pythonpath.py'
=== target is '../../_pythonpath.py'
=== removed symlink 'database/replication/_pythonpath.py'
=== target was '../schema/_pythonpath.py'
=== modified file 'database/schema/Makefile'
--- database/schema/Makefile	2009-04-17 10:32:16 +0000
+++ database/schema/Makefile	2009-05-29 00:29:45 +0000
@@ -2,7 +2,7 @@
 # One day the guts of this will be migrated to Python
 #
 
-PYTHON=python2.4
+PYTHON=../../bin/py
 
 # The database dump to restore on database creation
 SAMPLEDATA=../sampledata/current.sql

=== added symlink 'database/schema/_pythonpath.py'
=== target is '../../_pythonpath.py'
=== removed file 'database/schema/_pythonpath.py'
--- database/schema/_pythonpath.py	2008-09-02 10:25:32 +0000
+++ database/schema/_pythonpath.py	1970-01-01 00:00:00 +0000
@@ -1,13 +0,0 @@
-# Copyright 2004-2005 Canonical Ltd.  All rights reserved.
-"""
-Set the PYTHONPATH for database setup and maintenance scripts
-"""
-__metaclass__ = type
-
-import sys, os.path
-
-# Main lib directory.
-sys.path.insert(0, os.path.join(
-    os.path.dirname(__file__), os.pardir, os.pardir, 'lib',
-    ))
-

=== added symlink 'database/schema/pending/_pythonpath.py'
=== target is '../../../_pythonpath.py'
=== removed file 'database/schema/pending/_pythonpath.py'
--- database/schema/pending/_pythonpath.py	2005-11-17 21:44:26 +0000
+++ database/schema/pending/_pythonpath.py	1970-01-01 00:00:00 +0000
@@ -1,9 +0,0 @@
-# Copyright 2004-2005 Canonical Ltd.  All rights reserved.
-
-__metaclass__ = type
-
-import sys, os, os.path
-
-sys.path.insert(0, os.path.join(
-    os.path.dirname(__file__), os.pardir, os.pardir, os.pardir, 'lib'
-    ))

=== added file 'doc/buildout.txt'
--- doc/buildout.txt	1970-01-01 00:00:00 +0000
+++ doc/buildout.txt	2009-05-14 20:46:25 +0000
@@ -0,0 +1,586 @@
+Launchpad Buildout
+******************
+
+Launchpad is moving to using the buildout_ (or "zc.buildout") build system.  It
+will gradually replace the sourcecode directory, and hopefully also the
+Launchpad Makefile.
+
+Buildout's biggest strength is managing Python packages.  That will also be our
+primary focus for it.  Meanwhile, apt will continue to manage our Python
+language installation, as well some or all of our non-Python dependencies, such
+as PostgreSQL.  While Bazaar will continue to be an essential part of our
+toolchain, we will move away from using it to incorporate source code trees of
+our dependencies.
+
+If you are not interested in our `Motivations`_  or in an `Introduction to
+zc.buildout`_, all developers will at least want to read the very brief
+sections about the `Set Up`_ and the `Everyday Usage`_.
+
+Developers who manage source dependencies probably should read the general
+information about `Managing Dependencies and Scripts`_, but will also find
+detailed instructions to `Add a Package`_, to `Upgrade a Package`_, to `Add a
+Script`_, to `Add a File Modified By Buildout`_, and to `Work with Unreleased
+or Forked Packages`_.
+
+.. _buildout: http://www.buildout.org/
+
+===========
+Motivations
+===========
+
+These motivations are labeled as "[INTERNAL]" or "[EXTERNAL]" to indicate
+whether it is pertinent for internal dependencies, which we on the Launchpad
+team create and release ourselves; or external dependencies, which other
+parties, in and out of Canonical, create and release.  
+
+* We want more careful specification of our dependencies across branches.
+  [INTERNAL] [EXTERNAL]
+
+  This is a real concern pertinent both for our "trunks" (devel, stable,
+  db-devel, db-stable) and for our development boxes. For instance, now, in our
+  trunks, when we want to update a dependency, we need to make sure that *all*
+  the current Launchpad trunks work with the dependency initially; then submit
+  a new Launchpad branch that uses the change dependency.  A mistake can
+  even potentially break one or both of the db-* trunks, since PQM only tests
+  against one branch (usually devel), and sourcecode changes affect all
+  branches at once.  For simplicity, speed, and safety, we want to be able to
+  submit a single branch that incorporates the source dependencies and the
+  associated changes at once.
+
+  This is also true, if less critical and easier to work around, on developer
+  boxes.  Without care, changes to sourcecode when working on dependencies will
+  affect all a developer's branches, polluting test results with false
+  negatives or false positives.
+
+* We want to default to using released versions of our software dependencies.
+  [EXTERNAL]
+
+  A significant number of projects do not always have a pristine trunk, and
+  many also spend extra effort on polish, bug fixes, and compatibility before a
+  release.  If we do not desperately need a new feature on trunk, using a
+  release is generally regarded as a safer, better practice. Our current usage
+  of bzr branches of the development trunks does not encourage this practice.
+
+* We want to be encouraged to make the effort to interact with upstream
+  projects to have our patches integrated. [EXTERNAL]
+  
+  Interacting and negotiating with upstream is undeniably more time-consuming
+  than our current practice of maintaining local bzr branches with our patches,
+  especially short-term.  But our current practice is not good open-source
+  community behavior--an ironic characteristic for a project like Launchpad. It
+  also can cause problems down the road, for instance, if the patch becomes
+  stale and we want to migrate to new releases.
+
+* We want to be protected from changes and differences in our operating system.
+  [INTERNAL] [EXTERNAL]
+
+  This is a concern both over time and across different Launchpad environments.
+
+  First, our operating system, Ubuntu, is driven by many needs and goals.
+  Launchpad is among them, but generally Launchpad serves Ubuntu, not the
+  reverse.  For instance, recently Jaunty dropped Launchpad's Python version.
+  The Ubuntu developers had good reason--Python 2.4 has not been supported by
+  the Python developers for some time--but it caused a significant
+  inconvenience to the Launchpad team. Managing our dependencies, particularly
+  the Python library dependencies, can help alleviate these problems.
+
+  Second, Launchpad developers run a significantly different version of the
+  operating system than that run in production. Maintaining our Python library
+  dependencies ourselves can also help alleviate these concerns.
+
+* We want to be able to easily use standard packages of our primary
+  programming language, Python. [EXTERNAL]
+
+  Our Python library dependencies are distributed for many operating systems--
+  Windows, Mac, and other flavors of Linux--in a unified location and format:
+  PyPI, using distutils.  Using Python library dependencies in their standard
+  distributions makes it easier for us to reuse code.
+
+* We want to be encouraged to release Python packages of our open-source
+  code. [INTERNAL]
+
+  We are beginning to realize our aspirations of abstracting and releasing some
+  of our code.  Much of that code is in Python.  Consuming standard Python
+  packages encourages us to follow good practice in releasing our own Python
+  packages.  Dogfooding can help keep us honest, and good official releases can
+  help us support and encourage a community of users.
+
+Meanwhile, we want to be able to keep certain aspects of the legacy story.
+
+* We need deployment to not need network access.
+
+* We need to be able to use custom releases of our Python dependencies, if
+  absolutely necessary.
+
+* We would like to be able to follow security releases in our operating system.
+
+We will be able to support the first two of these aspects entirely.
+
+As to the third, we will continue to follow operating system security releases
+for most or all of the dependencies that are not Python packages--a very
+similar situation to the one before Buildout integration.
+
+===========================
+Introduction to zc.buildout
+===========================
+
+Buildout is a relatively simple system that increases in complexity as it is
+extended via "recipes".  It works on top of distutils_ and setuptools_.  It
+uses declarative ini-style files to direct its work.  The `Buildout site`_
+points to a variety of documents describing and documenting its use.
+
+For Launchpad, Buildout is hidden behind a Makefile as of this writing.  If
+the Makefile is removed, developers will typically run ``bootstrap.py`` in the
+branch, and then run ``bin/buildout``.  After that, the ``bin`` directory will
+have the commands to start, stop, test, and debug the software.  If you are
+interested in example direct usage of Buildout, you may want to read `the
+"Hacking" document in the Launchpad wiki`_ that describes the usage patterns in
+``lazr.*`` packages.
+
+.. _`the "Hacking" document in the Launchpad wiki`: https://dev.launchpad.net/Hacking
+
+Launchpad's Buildout usage is roughly of medium complexity.  It is more
+complex than that needed by a package with few dependencies and simple usage
+(see lazr.uri, for instance), but less complex than that of other large
+applications that use the buildout system.  More complexity can come by
+building more non-Python tools and by having multiple configuration variations,
+for instance.
+
+The documentation below will focus on using Launchpad's buildout.  See the
+links given above for a more thorough general review.
+
+.. _distutils: http://docs.python.org/distutils/index.html
+
+.. _setuptools: http://peak.telecommunity.com/DevCenter/setuptools
+
+.. _`Buildout site`: http://www.buildout.org/
+
+======
+Set Up
+======
+
+If you use the ``rocketfuel-get`` script, run that, and you will be done.
+
+If you don't, you have just a bit more initial set up.  I'll assume you
+maintain a pattern similar to what the ``rocketfuel-*`` scripts use: you have a
+local, pristine branch of trunk from which you make your other branches.  You
+manually update the trunk and rsync sourcecode when necessary.  When you make
+a branch, you use ``utilities/link-external-sourcecode``.
+
+Developers that take this approach should do the following, where ``trunk`` is
+the trunk branch from which you make local branches.
+
+::
+
+    bzr co lp:lp-source-dependencies download-cache
+
+Then run ``make`` in the trunk.
+
+See the `Everyday Usage: Manual`_ section for further instructions on how to
+work without the ``rocketfuel-*`` scripts.
+
+.. _`Everyday Usage: Manual`: Manual_
+
+==============
+Everyday Usage
+==============
+
+``rocketfuel`` Scripts
+======================
+
+If you typically use ``rocketfuel-get``, and you don't change source
+dependencies, you should not have any further changes, except that ``bin/test``
+has replaced ``test.py``.  ``rocketfuel-branch`` and
+``link-external-dependencies`` will Do the Right Thing.
+
+Manual
+======
+
+If you don't use the rocketfuel scripts, you will still use
+``link-external-dependencies`` as before.  When a buildout complains that it
+cannot find a version of a dependency, do the following, from within the
+branch::
+
+    bzr up download-cache
+
+After this, retry your make (or run ``bin/buildout`` from the branch).
+
+That's it for everyday usage.
+
+=================================
+Managing Dependencies and Scripts
+=================================
+
+What if you need to change or add dependencies or scripts?  As you might
+expect, you need to know a bit more about what's going on, although we can
+still keep this at a fairly high level.
+
+First let's talk a little about the anatomy of what we have set up.  To be
+clear, much of this is based on our own decisions of what to do.  If you see
+something problematic, bring it up with the Foundations team.  Maybe together
+we can come up with another approach that meets our needs better.
+
+If you saw the top-level Launchpad directory before we started using Buildout,
+you might notice seven new items in the checkout.
+
+``bootstrap.py``
+    This is the standard bootstrapping file provided by the Buildout
+    distribution.  As of this writing, the Makefile uses this file, and you do
+    not have to modify it or call it directly.
+    
+    Without a Makefile, the first step of developing a source tree that uses
+    Buildout is to run this file in the top level directory with the Python
+    executable that you wish to use for the source tree.  It creates a ``bin``
+    directory, if it does not already exist, and puts a ``buildout`` executable
+    there.  The next step is to run ``bin/buildout``, which, unless you supply
+    a ``-c`` option to specify a configuration file, will look for a
+    buildout.cfg file by default to discover what to do.
+    
+    Again, as of this writing, the Makefile uses this file, and it does not
+    need to be modified, so you need not concern yourself with it further at
+    this time.
+
+``ez_setup.py``
+    This is another standard file from another project.  In this case, it is
+    the file provided by the setuptools project to install setuptools.  It is
+    used by the ``setup.py`` file, described below.  It does not need to be
+    modified or called directly.
+
+``setup.py``
+    This is the file that uses ``distutils``, extended by ``setuptools``, to
+    specify direct dependencies, scripts, and other elements of the local
+    source tree.
+    
+    Buildout uses it, but the reverse is not true: ``setup.py`` does not know
+    about Buildout.  This means that packages that use Buildout for development
+    do not have to require it when they are being installed in other software
+    as a dependency.
+    
+    Describing this file in full is well beyond the scope of this document.  We
+    will give recipes for modifying it for certain tasks below. For more
+    information beyond these recipes, see the setuptools and distutils
+    documentation.
+
+``buildout.cfg``
+    This is the default configuration file that ``bin/buildout`` will look to
+    for instructions.
+
+    Describing it in full is well beyond the scope of this document.  However,
+    we will give an overview here.
+
+    Configuration files for Buildout are comprised of sections with key-value
+    pairs.
+
+    The key-value pairs are separated with new lines, when the subsequent line
+    is not indented.  The key and value are each separated with an equals sign.
+    
+    ::
+    
+        foo = bar
+        baz = bing
+              bah
+              boo
+        sha = zam
+
+    That example shows three keys, 'foo', 'baz', and 'sha'.  The 'baz' key has
+    a string with two new lines (which might be interpreted one several ways,
+    as defined for that key).
+
+    The ``[buildout]`` section is the starting point for Buildout to determine
+    what to do.  It looks for an ``extends`` key to find any additional files
+    to merge in; we use this for ``versions.cfg``, discussed below.
+    
+    In addition to general configuration and initialization such as this, it
+    looks in the ``develop`` key to find source trees to develop as part of the
+    buildout. In the standard Launchpad configuration, we develop only
+    Launchpad itself (the current directory, or '.').  This means that the
+    local ``setup.py`` will be run.  If you want to develop Launchpad while you
+    develop another dependency, you can link another source tree in, and
+    specify an additional ``develop`` directory in another line::
+    
+        [buildout]
+        develop = .
+                  lazr_uri_branch
+
+    The other basic key in the ``[buildout]`` section that we'll highlight here
+    is ``parts``.  This key identifies the other sections in buildout.cfg that
+    will be processed.  A section that is not identified in the ``[buildout]``
+    sections ``parts`` key will usually be ignored (unless chosen for another
+    role by another key elsewhere).
+    
+    Sections other than ``[buildout]`` that are specified as parts always must
+    specify a ``recipe``: an identifier that determines what code should
+    process that section.  You'll see a variety of recipes in Launchpad's
+    buildout.cfg, including ``z3c.recipe.filetemplate``, ``zc.recipe.egg``, and
+    others.
+
+``versions.cfg``
+    As mentioned above, ``buildout.cfg`` extends ``versions.cfg``  by
+    specifying it in the ``extends`` key of the ``[buildout]`` section.
+    Versions.cfg specifies the precise versions of the dependencies we use.
+    This means that we can have several versions of a dependency available
+    locally, but we only build the precise one we specify.  We give an
+    example of its use below.  To read about the mechanism used, see the
+    zc.buildout documentation of the ``versions`` option in the ``[buildout]``
+    section.
+
+``eggs``
+    The ``eggs`` directory holds the eggs built from the downloaded
+    distributions. Unless you set it up differently yourself, this directory is
+    shared by all your branches. This directory is local to your system--we do
+    not manage it in a branch. One reason for this is that eggs are often
+    platform-specific.
+
+``download-cache``
+    The ``download-cache`` directory is a set of downloaded distributions--that
+    is, exact copies of the items that would typically be obtained from the
+    Python Package Index ("PyPI"), or another download source. We manage the
+    download cache as a shared resource across all of our developers with a bzr
+    branch in a Launchpad project called ``lp-source-dependencies``.
+    
+    When we run buildout, Buildout reads a special key and value in the
+    ``[buildout]`` section: ``install-from-cache = true``.  This means that, by
+    default, Buildout will *not* use network access to find packages, but
+    *only* look in the download cache.  This has many advantages.  
+    
+    - First, it helps us keep our deployment boxes from needing network access
+      out to PyPI and other download sites.
+
+    - Second, it makes the buildout much faster, because it does not have to
+      look out on the net for every dependency. 
+
+    - Third, it makes the buildout more repeatable, because we are more
+      insulated from outages at download sites such as PyPI, and poor release
+      management. 
+
+    - Fourth, it makes our deployments more auditable, because we can tell
+      exactly what we are deploying.
+
+    - Fifth, it gives us a single obvious place to put custom package
+      distributions, as we'll discuss below.
+
+    The downside is that adding and upgrading packages takes a small additional
+    step, as we'll see below.
+
+``buildout-templates``
+    The last additional item in the checkout is the ``buildout-templates``
+    directory.  This is used to hold templates that are used by the
+    section in buildout.cfg that uses the ``z3c.recipe.filetemplate`` recipe.
+    This can be used for many things, but we are using it as an alternate way
+    for producing scripts when the zc.recipe.egg approach is insufficient.
+
+In addition to these seven listings, after you have run the Makefile (or
+``bin/buildout``), you will see an additional listing:
+
+``bin``
+    The ``bin`` directory has already been discussed many times.  After running
+    the bootstrap.py, it holds the ``buildout`` script which can be used to
+    process Buildout configuration files.  In Launchpad's case, after running
+    the buildout, it also holds many executables, including scripts to test
+    Launchpad; to run it; to run Python or IPython with Launchpad's sourcetree
+    and dependencies available; to run harness or iharness (with IPython) with
+    the sourcetree, dependencies, and database connections; or to perform
+    several other tasks.  For now, the Makefile provides aliases for many of
+    these.
+
+Now that you have an introduction to the pertinent files and directories, we'll
+move on to trying to perform tasks in the buildout.  We'll discuss adding a
+dependency, upgrading a dependency, adding a script, adding an arbitrary file,
+and working with unreleased packages.
+
+Add a Package
+=============
+
+Let's suppose that we want to add the "lazr.foo" package as a dependency.
+
+1.  Add the new package to the ``setup.py`` file in the ``install_requires``
+    list.
+    
+    Generally, our policy is to only set minimum version numbers in this file,
+    or none at all.  It doesn't really matter for an application like
+    Launchpad, but it a good rule for library packages, so we follow it for
+    consistency.  Therefore, we might simply add ``'lazr.foo'`` to
+    install_requires, or ``'lazr.foo >= 1.1'`` if we know that we are depending
+    on features introduced in version 1.1 of lazr.foo.
+
+2.  [OPTIONAL] If you know it, add the desired version number to versions.cfg
+    now.
+    
+    For instance, if you know you want lazr.foo 1.1.2, add this line to the
+    ``[versions]`` section of ``versions.cfg``::
+  
+      lazr.foo = 1.1.2
+
+3.  [OPTIONAL] Add the desired distribution of lazr.foo 1.1.2 to the
+    ``download-cache/dist`` directory.
+
+4.  Run the following command (or your variation)::
+
+        ./bin/buildout -v buildout:install-from-cache=false | tee out.txt | grep 'Picked'
+
+    The first part (``./bin/buildout -v buildout:install-from-cache=false``)
+    will run buildout, allowing it to download source packages from the
+    Internet to ``download-cache/dist``. The second part (``tee out.txt``) will
+    dump the full output to the ``out.txt`` file in case you need to debug a
+    problem. The last part (``grep 'Picked'``) will filter the output so that
+    only additional packages--dependencies of your dependency--will be listed.
+    Look at the output. You need to see if it means that you have dependencies,
+    some of which may be indirect dependencies. Here's an imaginary example
+    output::
+
+        Picked: setuptools 0.6c9    
+        Picked: lazr.foom = 1.4
+        Picked: zope.bar = 3.6.1
+        Picked: z3c.shazam = 2.0.1
+
+    At this time, note that the output will include at least one, and possibly
+    more, spurious "Picked:" listings.  setuptools, in particular, shows up
+    repeatedly.
+    
+    In our example, other than the spurious ``setuptools`` listing, this means
+    that these packages have also been added to your ``download-cache/dist``
+    directory. You also need to add those versions to the ``versions.conf``
+    file::
+    
+        lazr.foom = 1.4
+        zope.bar = 3.6.1
+        z3c.shazam = 2.0.1
+
+5.  Test.
+
+    [TODO] Note that you can tell ec2test to include all uncommitted
+    distributions from the local download-cache in its tests.  If you do this,
+    you cannot use the ec2test feature to submit on test success.  Also, if you
+    have uncommitted distributions and you do *not* explicitly tell ec2test to
+    include or ignore the uncommitted distributions, it will refuse to start
+    an instance.
+
+6.  Commit the changes (``cd download-cache``, ``bzr up``,
+    ``bzr commit -m 'Add lazr.foom 1.1.2 and depdendencies to the download
+    cache'``) to the shared download cache when you are sure it is what you
+    want.
+
+You should only add packages to the download-cache.
+
+*Never* modify a package in the download-cache.
+
+Only remove a package if you have explicit permission to do so.
+
+Upgrade a Package
+=================
+
+Sometimes you need to upgrade a dependency.  This may require additional
+dependency additions or upgrades.
+
+If you already know what version you want, the simplest thing to try is to
+modify versions.cfg to specify the new version and run steps 4, 5, and 6 of the
+`Add a Package`_ instructions.
+
+If you don't know what version you want, but just want to see what happens when
+you upgrade to the most recent revision, you can clear out the versions of the
+packages for upgrade and give it a try (that is, run steps 4, 5, and 6 of the
+`Add a Package`_ instructions).  Note that, when not given an explicit version
+number, our buildout is set to prefer final releases over alpha and beta
+releases.  If you want to temporarily override this behavior, include
+``buildout:prefer-final=false`` as another argument to ``bin/buildout``.
+
+Add a Script
+============
+
+We often need scripts that are run in a certain environment defined by Python
+dependencies, and sometimes even different Python executables.  Several of the
+scripts we have are specified using the setuptools-based spelling that the
+``zc.recipe.egg`` recipe supports.
+
+For the common case, in ``setup.py``, add a string in the ``console_scripts``
+list of the ``entry_points`` argument. Here's an example string::
+
+    'run = canonical.launchpad.scripts.runlaunchpad:start_launchpad'
+
+This will create a script named ``run`` in the ``bin`` directory that calls the
+``start_launchpad`` function in the
+``canonical.launchpad.scripts.runlaunchpad`` module.
+
+See the `zc.recipe.egg documentation`_ for more information on how to add
+scripts using this method.
+
+.. _`zc.recipe.egg documentation`: http://pypi.python.org/pypi/zc.recipe.egg
+
+Add a File Modified By Buildout
+===============================
+
+Sometimes we need more control for the way our scripts are generated, or we
+need other files processed during a buildout.  Writing a custom zc.buildout
+recipe is one way, but well out of the scope of this document.  Read the
+zc.buildout documentation for direction.
+
+A much easier, and more limited approach is to use `z3c.recipe.filetemplate`_
+to build the file.  In the Launchpad configuration, this is used in the
+``[filetemplates]`` section, and is configured to use the files in the
+``buildout-templates`` directory for sources.  Basically, it finds .in files
+and installs them as found in the source directory, processing them with the
+values from the buildout configuration.
+
+.. _`z3c.recipe.filetemplate`: http://pypi.python.org/pypi/z3c.recipe.filetemplate
+
+Work with Unreleased or Forked Packages
+=======================================
+
+Sometimes you need to work with unreleased or forked packages.  FeedValidator_,
+for instance, makes nightly zip releases but other than that only offers svn
+access.  Similarly, we may require a patched or unreleased version of a package
+for some purpose.  Hopefully, these situations will be rare, but they do occur.
+
+While `other answers`_ are available for Buildout, our solution is to use the
+download-cache.  Basically, make a custom source distribution with a unique
+suffix in the name, and use it (and its version name) for the normal process of
+adding or updating a package, as described above.  Because the custom package
+is in the download-cache, it will be found and used.
+
+Here's an example of making a custom distribution of FeedValidator.
+
+FeedValidator is a Subversion project.  We check it out::
+
+    svn co http://feedvalidator.googlecode.com/svn/trunk/feedvalidator/src feedvalidator
+
+Next, we ``cd feedvalidator``, and, using a Python that has setuptools
+installed, we run the following command::
+
+    python setup.py egg_info -r -bDEV sdist
+
+For this example, imagine that the current revision of the repository is 1049.
+Because setuptools has built-in Subversion support, the command above will
+create a tar.gz in the ``dist`` directory named
+``feedvalidator-0.0.0DEV-r1049.tar.gz``. The -r option specifies that the
+subversion revision should be part of the package name.  The -bDEV option
+specifies that the 'DEV' suffix should be added to the version number.
+
+We could then put the tar.gz file in Launchpad's ``download-cache/dist``
+directory, specify ``feedvalidator = 0.0.0DEV-r1049`` in the ``versions.cfg``
+file, and proceed with the usual steps to update or add a new package.
+
+If you use a bzr branch, you might use the ``-d`` option instead of the ``-r``
+option when you create the distribution.  This will add the date instead of the
+revision::
+
+    python setup.py egg_info -d -bDEV sdist
+
+For instance, this might produce a distribution for the ``lazr.restful``
+project with a name like this: ``lazr.restful-0.9.1DEV-20090512.tar.gz``.
+
+See the setuptools documentation for more information about `the egg_info
+command`_.
+
+.. _FeedValidator: http://feedvalidator.org/
+
+.. _`other answers`: http://pypi.python.org/pypi/zc.buildoutsftp
+
+.. _`the egg_info command`: http://peak.telecommunity.com/DevCenter/setuptools#tagging-and-daily-build-or-snapshot-releases
+
+=====================
+Possible Future Goals
+=====================
+
+- No longer use system site-packages.
+- No longer use make.
+- Get rid of the sourcecode directory.

=== added file 'ez_setup.py'
--- ez_setup.py	1970-01-01 00:00:00 +0000
+++ ez_setup.py	2009-05-31 22:17:45 +0000
@@ -0,0 +1,280 @@
+#!python
+
+# NOTE TO LAUNCHPAD DEVELOPERS: This is a bootstrapping file from the
+# setuptools project.  It is imported by our setup.py.
+
+#!python
+"""Bootstrap setuptools installation
+
+If you want to use setuptools in your package's setup.py, just include this
+file in the same directory with it, and add this to the top of your setup.py::
+
+    from ez_setup import use_setuptools
+    use_setuptools()
+
+If you want to require a specific version of setuptools, set a download
+mirror, or use an alternate download directory, you can do so by supplying
+the appropriate options to ``use_setuptools()``.
+
+This file can also be run as a script to install or upgrade setuptools.
+"""
+import sys
+DEFAULT_VERSION = "0.6c9"
+DEFAULT_URL     = "http://pypi.python.org/packages/%s/s/setuptools/" % sys.version[:3]
+
+md5_data = {
+    'setuptools-0.6b1-py2.3.egg': '8822caf901250d848b996b7f25c6e6ca',
+    'setuptools-0.6b1-py2.4.egg': 'b79a8a403e4502fbb85ee3f1941735cb',
+    'setuptools-0.6b2-py2.3.egg': '5657759d8a6d8fc44070a9d07272d99b',
+    'setuptools-0.6b2-py2.4.egg': '4996a8d169d2be661fa32a6e52e4f82a',
+    'setuptools-0.6b3-py2.3.egg': 'bb31c0fc7399a63579975cad9f5a0618',
+    'setuptools-0.6b3-py2.4.egg': '38a8c6b3d6ecd22247f179f7da669fac',
+    'setuptools-0.6b4-py2.3.egg': '62045a24ed4e1ebc77fe039aa4e6f7e5',
+    'setuptools-0.6b4-py2.4.egg': '4cb2a185d228dacffb2d17f103b3b1c4',
+    'setuptools-0.6c1-py2.3.egg': 'b3f2b5539d65cb7f74ad79127f1a908c',
+    'setuptools-0.6c1-py2.4.egg': 'b45adeda0667d2d2ffe14009364f2a4b',
+    'setuptools-0.6c2-py2.3.egg': 'f0064bf6aa2b7d0f3ba0b43f20817c27',
+    'setuptools-0.6c2-py2.4.egg': '616192eec35f47e8ea16cd6a122b7277',
+    'setuptools-0.6c3-py2.3.egg': 'f181fa125dfe85a259c9cd6f1d7b78fa',
+    'setuptools-0.6c3-py2.4.egg': 'e0ed74682c998bfb73bf803a50e7b71e',
+    'setuptools-0.6c3-py2.5.egg': 'abef16fdd61955514841c7c6bd98965e',
+    'setuptools-0.6c4-py2.3.egg': 'b0b9131acab32022bfac7f44c5d7971f',
+    'setuptools-0.6c4-py2.4.egg': '2a1f9656d4fbf3c97bf946c0a124e6e2',
+    'setuptools-0.6c4-py2.5.egg': '8f5a052e32cdb9c72bcf4b5526f28afc',
+    'setuptools-0.6c5-py2.3.egg': 'ee9fd80965da04f2f3e6b3576e9d8167',
+    'setuptools-0.6c5-py2.4.egg': 'afe2adf1c01701ee841761f5bcd8aa64',
+    'setuptools-0.6c5-py2.5.egg': 'a8d3f61494ccaa8714dfed37bccd3d5d',
+    'setuptools-0.6c6-py2.3.egg': '35686b78116a668847237b69d549ec20',
+    'setuptools-0.6c6-py2.4.egg': '3c56af57be3225019260a644430065ab',
+    'setuptools-0.6c6-py2.5.egg': 'b2f8a7520709a5b34f80946de5f02f53',
+    'setuptools-0.6c7-py2.3.egg': '209fdf9adc3a615e5115b725658e13e2',
+    'setuptools-0.6c7-py2.4.egg': '5a8f954807d46a0fb67cf1f26c55a82e',
+    'setuptools-0.6c7-py2.5.egg': '45d2ad28f9750e7434111fde831e8372',
+    'setuptools-0.6c8-py2.3.egg': '50759d29b349db8cfd807ba8303f1902',
+    'setuptools-0.6c8-py2.4.egg': 'cba38d74f7d483c06e9daa6070cce6de',
+    'setuptools-0.6c8-py2.5.egg': '1721747ee329dc150590a58b3e1ac95b',
+    'setuptools-0.6c9-py2.3.egg': 'a83c4020414807b496e4cfbe08507c03',
+    'setuptools-0.6c9-py2.4.egg': '260a2be2e5388d66bdaee06abec6342a',
+    'setuptools-0.6c9-py2.5.egg': 'fe67c3e5a17b12c0e7c541b7ea43a8e6',
+    'setuptools-0.6c9-py2.6.egg': 'ca37b1ff16fa2ede6e19383e7b59245a',
+}
+
+import sys, os
+try: from hashlib import md5
+except ImportError: from md5 import md5
+
+def _validate_md5(egg_name, data):
+    if egg_name in md5_data:
+        digest = md5(data).hexdigest()
+        if digest != md5_data[egg_name]:
+            print >>sys.stderr, (
+                "md5 validation of %s failed!  (Possible download problem?)"
+                % egg_name
+            )
+            sys.exit(2)
+    return data
+
+def use_setuptools(
+    version=DEFAULT_VERSION, download_base=DEFAULT_URL, to_dir=os.curdir,
+    download_delay=15
+):
+    """Automatically find/download setuptools and make it available on sys.path
+
+    `version` should be a valid setuptools version number that is available
+    as an egg for download under the `download_base` URL (which should end with
+    a '/').  `to_dir` is the directory where setuptools will be downloaded, if
+    it is not already available.  If `download_delay` is specified, it should
+    be the number of seconds that will be paused before initiating a download,
+    should one be required.  If an older version of setuptools is installed,
+    this routine will print a message to ``sys.stderr`` and raise SystemExit in
+    an attempt to abort the calling script.
+    """
+    was_imported = 'pkg_resources' in sys.modules or 'setuptools' in sys.modules
+    def do_download():
+        egg = download_setuptools(version, download_base, to_dir, download_delay)
+        sys.path.insert(0, egg)
+        import setuptools; setuptools.bootstrap_install_from = egg
+    try:
+        import pkg_resources
+    except ImportError:
+        return do_download()       
+    try:
+        pkg_resources.require("setuptools>="+version); return
+    except pkg_resources.VersionConflict, e:
+        if was_imported:
+            print >>sys.stderr, (
+            "The required version of setuptools (>=%s) is not available, and\n"
+            "can't be installed while this script is running. Please install\n"
+            " a more recent version first, using 'easy_install -U setuptools'."
+            "\n\n(Currently using %r)"
+            ) % (version, e.args[0])
+            sys.exit(2)
+        else:
+            del pkg_resources, sys.modules['pkg_resources']    # reload ok
+            return do_download()
+    except pkg_resources.DistributionNotFound:
+        return do_download()
+
+def download_setuptools(
+    version=DEFAULT_VERSION, download_base=DEFAULT_URL, to_dir=os.curdir,
+    delay = 15
+):
+    """Download setuptools from a specified location and return its filename
+
+    `version` should be a valid setuptools version number that is available
+    as an egg for download under the `download_base` URL (which should end
+    with a '/'). `to_dir` is the directory where the egg will be downloaded.
+    `delay` is the number of seconds to pause before an actual download attempt.
+    """
+    import urllib2, shutil
+    egg_name = "setuptools-%s-py%s.egg" % (version,sys.version[:3])
+    url = download_base + egg_name
+    saveto = os.path.join(to_dir, egg_name)
+    src = dst = None
+    if not os.path.exists(saveto):  # Avoid repeated downloads
+        try:
+            from distutils import log
+            if delay:
+                log.warn("""
+---------------------------------------------------------------------------
+This script requires setuptools version %s to run (even to display
+help).  I will attempt to download it for you (from
+%s), but
+you may need to enable firewall access for this script first.
+I will start the download in %d seconds.
+
+(Note: if this machine does not have network access, please obtain the file
+
+   %s
+
+and place it in this directory before rerunning this script.)
+---------------------------------------------------------------------------""",
+                    version, download_base, delay, url
+                ); from time import sleep; sleep(delay)
+            log.warn("Downloading %s", url)
+            src = urllib2.urlopen(url)
+            # Read/write all in one block, so we don't create a corrupt file
+            # if the download is interrupted.
+            data = _validate_md5(egg_name, src.read())
+            dst = open(saveto,"wb"); dst.write(data)
+        finally:
+            if src: src.close()
+            if dst: dst.close()
+    return os.path.realpath(saveto)
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+def main(argv, version=DEFAULT_VERSION):
+    """Install or upgrade setuptools and EasyInstall"""
+    try:
+        import setuptools
+    except ImportError:
+        egg = None
+        try:
+            egg = download_setuptools(version, delay=0)
+            sys.path.insert(0,egg)
+            from setuptools.command.easy_install import main
+            return main(list(argv)+[egg])   # we're done here
+        finally:
+            if egg and os.path.exists(egg):
+                os.unlink(egg)
+    else:
+        if setuptools.__version__ == '0.0.1':
+            print >>sys.stderr, (
+            "You have an obsolete version of setuptools installed.  Please\n"
+            "remove it from your system entirely before rerunning this script."
+            )
+            sys.exit(2)
+
+    req = "setuptools>="+version
+    import pkg_resources
+    try:
+        pkg_resources.require(req)
+    except pkg_resources.VersionConflict:
+        try:
+            from setuptools.command.easy_install import main
+        except ImportError:
+            from easy_install import main
+        main(list(argv)+[download_setuptools(delay=0)])
+        sys.exit(0) # try to force an exit
+    else:
+        if argv:
+            from setuptools.command.easy_install import main
+            main(argv)
+        else:
+            print "Setuptools version",version,"or greater has been installed."
+            print '(Run "ez_setup.py -U setuptools" to reinstall or upgrade.)'
+
+def update_md5(filenames):
+    """Update our built-in md5 registry"""
+
+    import re
+
+    for name in filenames:
+        base = os.path.basename(name)
+        f = open(name,'rb')
+        md5_data[base] = md5(f.read()).hexdigest()
+        f.close()
+
+    data = ["    %r: %r,\n" % it for it in md5_data.items()]
+    data.sort()
+    repl = "".join(data)
+
+    import inspect
+    srcfile = inspect.getsourcefile(sys.modules[__name__])
+    f = open(srcfile, 'rb'); src = f.read(); f.close()
+
+    match = re.search("\nmd5_data = {\n([^}]+)}", src)
+    if not match:
+        print >>sys.stderr, "Internal error!"
+        sys.exit(2)
+
+    src = src[:match.start(1)] + repl + src[match.end(1):]
+    f = open(srcfile,'w')
+    f.write(src)
+    f.close()
+
+
+if __name__=='__main__':
+    if len(sys.argv)>2 and sys.argv[1]=='--md5update':
+        update_md5(sys.argv[2:])
+    else:
+        main(sys.argv[1:])
+
+
+
+
+

=== modified file 'lib/canonical/database/harness.py'
--- lib/canonical/database/harness.py	2009-05-12 08:16:10 +0000
+++ lib/canonical/database/harness.py	2009-05-29 00:39:22 +0000
@@ -1,24 +1,22 @@
-#! /usr/bin/python2.4 -i
-
-# This is designed to be used as follows:
-#
-#   python -i harness.py
-#
-# At that point, you will have the Launchpad Storm classes, all interface
-# classes and the zope3 CA-fu at your fingertips, connected to launchpad_dev
-# or your LP_DBNAME environment variable (if you have one set).
+# Copyright 2009 Canonical Ltd.  All rights reserved.
+
+"""Scripts for starting a Python prompt with Launchpad initialized.
+
+The scripts provide an interactive prompt with the Launchpad Storm classes,
+all interface classes and the zope3 CA-fu at your fingertips, connected to
+launchpad_dev or your LP_DBNAME environment variable (if you have one set).
+One uses Python, the other iPython.
+"""
+
+__metaclass__ = type
+__all__ = ['python', 'ipython']
+
+# This has setup.py scripts.  It is usually installed via buildout.
+#
+
 #
 import os
 import sys
-#sys.path.insert(0, '../sourcecode/zope/src/')
-#sys.path.insert(0, '../sourcecode/sqlobject/')
-sys.path.insert(0, '../..')
-#sys.path.insert(0, '..')
-
-if len(sys.argv) > 1:
-    dbuser = sys.argv[1]
-else:
-    dbuser = None
 
 import transaction
 
@@ -27,26 +25,6 @@
 
 from canonical.launchpad.scripts import execute_zcml_for_scripts
 
-print 'execute_zcml_for_scripts()...'
-execute_zcml_for_scripts()
-print 'xmlconfig.file()...'
-xmlconfig.file('script.zcml', execute=True)
-
-#
-# setup connection to the db
-#
-#from canonical.lp import initZopeless
-#transactionmgr = initZopeless(dbuser=dbuser)
-
-def switch_db_user(dbuser, commit_first=True):
-    global transactionmgr
-    if commit_first:
-        transactionmgr.commit()
-    else:
-        transactionmgr.abort()
-    transactionmgr.uninstall()
-    transactionmgr = initZopeless(dbuser=dbuser)
-
 #
 # We don't really depend on everything from canonical.launchpad.database and
 # canonical.launchpad.interfaces, but it's good to have this available in the
@@ -64,13 +42,6 @@
 
 import readline
 import rlcompleter
-readline.parse_and_bind('tab: complete')
-
-# Mimic the real interactive interpreter's loading of any $PYTHONSTARTUP file.
-print 'Reading $PYTHONSTARTUP...'
-startup = os.environ.get('PYTHONSTARTUP')
-if startup:
-    execfile(startup)
 
 # Bring in useful bits of Storm.
 from storm.locals import *
@@ -79,23 +50,64 @@
         IStoreSelector, MAIN_STORE, AUTH_STORE, MASTER_FLAVOR,
         SLAVE_FLAVOR, DEFAULT_FLAVOR)
 
-print 'Initializing storm...'
-store_selector = getUtility(IStoreSelector)
-store = store_selector.get(MAIN_STORE, MASTER_FLAVOR)
-
-# Let's get a few handy objects going.
-if dbuser == 'launchpad':
-    print 'Creating a few handy objects...'
-    d = Distribution.get(1)
-    p = Person.get(1)
-    ds = DistroSeries.get(1)
-    prod = Product.get(1)
-    proj = Project.get(1)
-    b2 = Bug.get(2)
-    b1 = Bug.get(1)
-    s = Specification.get(1)
-    q = Question.get(1)
-
-# Having a factory instance is handy.
-print 'Creating the factory...'
-factory = LaunchpadObjectFactory()
+
+def switch_db_user(dbuser, commit_first=True):
+    global transactionmgr
+    if commit_first:
+        transactionmgr.commit()
+    else:
+        transactionmgr.abort()
+    transactionmgr.uninstall()
+    transactionmgr = initZopeless(dbuser=dbuser)
+
+
+def _get_locals():
+    if len(sys.argv) > 1:
+        dbuser = sys.argv[1]
+    else:
+        dbuser = None
+    print 'execute_zcml_for_scripts()...'
+    execute_zcml_for_scripts()
+    print 'xmlconfig.file()...'
+    xmlconfig.file('script.zcml', execute=True)
+    readline.parse_and_bind('tab: complete')
+    # Mimic the real interactive interpreter's loading of any $PYTHONSTARTUP file.
+    print 'Reading $PYTHONSTARTUP...'
+    startup = os.environ.get('PYTHONSTARTUP')
+    if startup:
+        execfile(startup)
+    print 'Initializing storm...'
+    store_selector = getUtility(IStoreSelector)
+    store = store_selector.get(MAIN_STORE, MASTER_FLAVOR)
+
+    # Let's get a few handy objects going.
+    if dbuser == 'launchpad':
+        print 'Creating a few handy objects...'
+        d = Distribution.get(1)
+        p = Person.get(1)
+        ds = DistroSeries.get(1)
+        prod = Product.get(1)
+        proj = Project.get(1)
+        b2 = Bug.get(2)
+        b1 = Bug.get(1)
+        s = Specification.get(1)
+        q = Question.get(1)
+
+    # Having a factory instance is handy.
+    print 'Creating the factory...'
+    factory = LaunchpadObjectFactory()
+    res = {}
+    res.update(locals())
+    res.update(globals())
+    del res['_get_locals']
+    return res
+
+
+def python():
+    import code
+    code.interact(banner='', local=_get_locals())
+
+
+def ipython():
+    import IPython.ipapi
+    IPython.ipapi.launch_new_instance(_get_locals())

=== modified file 'lib/canonical/launchpad/mailman/runmailman.py'
--- lib/canonical/launchpad/mailman/runmailman.py	2008-09-23 21:54:47 +0000
+++ lib/canonical/launchpad/mailman/runmailman.py	2009-05-28 19:05:53 +0000
@@ -47,6 +47,10 @@
         stderr = None
     env = dict(os.environ)
     env['LPCONFIG'] = config.instance_name
+    # We want to make sure that the Launchpad process will have the benefit of
+    # all of the dependency paths inserted by the buildout bin/run script. We
+    # pass them via PYTHONPATH.
+    env['PYTHONPATH'] = os.path.pathsep.join(sys.path)
     process = subprocess.Popen(
         args, cwd=mailman_bin, stdout=stdout, stderr=stderr, env=env)
     code = process.wait()

=== modified file 'lib/canonical/launchpad/scripts/runlaunchpad.py'
--- lib/canonical/launchpad/scripts/runlaunchpad.py	2009-01-09 02:59:19 +0000
+++ lib/canonical/launchpad/scripts/runlaunchpad.py	2009-05-13 00:29:48 +0000
@@ -92,6 +92,12 @@
             "--logfile", logfile,
             ]
 
+        # We want to make sure that the Launchpad process will have the benefit
+        # of all of the dependency paths inserted by the buildout bin/run
+        # script. We pass them via PYTHONPATH.
+        env = dict(os.environ)
+        env['PYTHONPATH'] = os.path.pathsep.join(sys.path)
+
         if config[self.section_name].spew:
             args.append("--spew")
 
@@ -100,7 +106,7 @@
         # log to stdout and redirect it ourselves because we then lose the
         # ability to cycle the log files by sending a signal to the twisted
         # process.
-        process = subprocess.Popen(args, stdin=subprocess.PIPE)
+        process = subprocess.Popen(args, stdin=subprocess.PIPE, env=env)
         process.stdin.close()
         # I've left this off - we still check at termination and we can
         # avoid the startup delay. -- StuartBishop 20050525

=== modified file 'lib/canonical/launchpad/scripts/tests/loglevels.py'
--- lib/canonical/launchpad/scripts/tests/loglevels.py	2009-03-27 03:29:31 +0000
+++ lib/canonical/launchpad/scripts/tests/loglevels.py	2009-05-29 16:22:44 +0000
@@ -8,12 +8,6 @@
 __metaclass__ = type
 __all__ = []
 
-# Fix path so imports work.
-import sys, os, os.path
-sys.path.insert(0, os.path.join(
-    os.path.dirname(__file__), os.pardir, os.pardir, os.pardir, os.pardir,
-    ))
-
 # Monkey patch time.gmtime to make our tests easier to read.
 import time
 def fake_gmtime(ignored_seconds):

=== modified file 'lib/canonical/launchpad/scripts/tests/test_logger.txt'
--- lib/canonical/launchpad/scripts/tests/test_logger.txt	2009-03-27 03:29:31 +0000
+++ lib/canonical/launchpad/scripts/tests/test_logger.txt	2009-05-29 20:30:34 +0000
@@ -3,10 +3,17 @@
 by our logging handler.
 
     >>> def test(*args, **kw):
+    ...     import os
     ...     import sys
     ...     import subprocess
-    ...     import os.path
     ...     from canonical.config import config
+    ...     # We want to make sure that the subprocess will have the benefit
+    ...     # of all of the dependency paths inserted by the buildout.
+    ...     # We pass them via PYTHONPATH.
+    ...     env = kw.get('env')
+    ...     if env is None:
+    ...         kw['env'] = env = dict(os.environ)
+    ...     env['PYTHONPATH'] = os.path.pathsep.join(sys.path)
     ...     test_script_path = os.path.join(
     ...         config.root, 'lib', 'canonical', 'launchpad',
     ...         'scripts', 'tests', 'loglevels.py')

=== removed symlink 'lib/launchpadlib'
=== target was '../sourcecode/launchpadlib/src/launchpadlib'
=== modified file 'lib/lazr/__init__.py'
--- lib/lazr/__init__.py	2008-12-19 20:41:31 +0000
+++ lib/lazr/__init__.py	2009-05-28 18:51:07 +0000
@@ -1,2 +1,10 @@
 # This is a namespace package for lazr.* packages.  We should absolutely get rid
 # of this once we are using eggs.
+import warnings
+warnings.filterwarnings(
+    'ignore',
+    'Module .+ was already imported from .+, but .+ is being added.*',
+    UserWarning)
+
+import pkg_resources
+pkg_resources.declare_namespace('lazr')

=== removed symlink 'lib/lazr/uri'
=== target was '../../sourcecode/lazr.uri/src/lazr/uri'
=== added directory 'lib/lp/scripts'
=== added file 'lib/lp/scripts/__init__.py'
=== added directory 'lib/lp/scripts/utilities'
=== added file 'lib/lp/scripts/utilities/__init__.py'
=== added file 'lib/lp/scripts/utilities/apiindex.py'
--- lib/lp/scripts/utilities/apiindex.py	1970-01-01 00:00:00 +0000
+++ lib/lp/scripts/utilities/apiindex.py	2009-05-29 20:30:34 +0000
@@ -0,0 +1,20 @@
+# Copyright 2009 Canonical Ltd.  All rights reserved.
+
+"""A script to build reference html from wadl.
+
+Uses an xsl from the launchpadlib package."""
+
+__metaclass__ = type
+__all__ = ['main']
+
+
+import pkg_resources
+import subprocess
+import sys
+
+def main():
+    source = sys.argv[1]
+    stylesheet = pkg_resources.resource_filename(
+        'launchpadlib', 'wadl-to-refhtml.xsl')
+    subprocess.call(['xsltproc', stylesheet, source])
+    pkg_resources.cleanup_resources()
\ No newline at end of file

=== renamed file 'importfascist.py' => 'lib/lp/scripts/utilities/importfascist.py'
=== renamed file 'utilities/killservice.py' => 'lib/lp/scripts/utilities/killservice.py'
--- utilities/killservice.py	2008-09-23 15:37:03 +0000
+++ lib/lp/scripts/utilities/killservice.py	2009-05-01 00:25:52 +0000
@@ -1,13 +1,10 @@
-#!/usr/bin/python2.4
+#!../bin/py
 
 # Copyright 2004-2005 Canonical Ltd.  All rights reserved.
 # This module uses relative imports.
 # pylint: disable-msg=W0403
-
 __metaclass__ = type
 
-import _pythonpath
-
 import os, logging
 from signal import SIGTERM
 from optparse import OptionParser
@@ -17,7 +14,7 @@
 from canonical.launchpad.mailman.runmailman import stop_mailman
 
 
-if __name__ == '__main__':
+def main():
     parser = OptionParser('Usage: %prog [options] [SERVICE ...]')
     logger_options(parser, logging.INFO)
     (options, args) = parser.parse_args()

=== renamed file 'warninghandler.py' => 'lib/lp/scripts/utilities/warninghandler.py'
=== removed symlink 'lib/oauth'
=== target was '../sourcecode/oauth/oauth'
=== removed symlink 'lib/openid'
=== target was '../sourcecode/openid/openid'
=== removed symlink 'lib/wadllib'
=== target was '../sourcecode/wadllib/src/wadllib'
=== removed file 'scripts/_pythonpath.py'
--- scripts/_pythonpath.py	2008-06-19 07:57:48 +0000
+++ scripts/_pythonpath.py	1970-01-01 00:00:00 +0000
@@ -1,12 +0,0 @@
-# Copyright 2004-2005 Canonical Ltd.  All rights reserved.
-
-__metaclass__ = type
-
-import sys, os, os.path
-
-sys.path.insert(0, os.path.join(
-    os.path.dirname(__file__), os.pardir, 'lib'
-    ))
-
-# Enable Storm's C extensions
-os.environ['STORM_CEXTENSIONS'] = '1'

=== added symlink 'scripts/_pythonpath.py'
=== target is '../_pythonpath.py'
=== added symlink 'scripts/ftpmaster-tools/_pythonpath.py'
=== target is '../../_pythonpath.py'
=== removed file 'scripts/ftpmaster-tools/_pythonpath.py'
--- scripts/ftpmaster-tools/_pythonpath.py	2008-06-19 07:57:48 +0000
+++ scripts/ftpmaster-tools/_pythonpath.py	1970-01-01 00:00:00 +0000
@@ -1,12 +0,0 @@
-# Copyright 2004-2005 Canonical Ltd.  All rights reserved.
-
-__metaclass__ = type
-
-import sys, os
-
-sys.path.insert(0, os.path.join(
-    os.path.dirname(__file__), os.pardir, os.pardir, 'lib'))
-
-# Enable Storm's C extensions
-os.environ['STORM_CEXTENSIONS'] = '1'
-

=== removed file 'scripts/rosetta/_pythonpath.py'
--- scripts/rosetta/_pythonpath.py	2008-06-19 07:57:48 +0000
+++ scripts/rosetta/_pythonpath.py	1970-01-01 00:00:00 +0000
@@ -1,12 +0,0 @@
-# Copyright 2004-2005 Canonical Ltd.  All rights reserved.
-
-__metaclass__ = type
-
-import sys, os, os.path
-
-sys.path.insert(0, os.path.join(
-    os.path.dirname(__file__), os.pardir, os.pardir, 'lib'
-    ))
-
-# Enable Storm's C extensions
-os.environ['STORM_CEXTENSIONS'] = '1'

=== added symlink 'scripts/rosetta/_pythonpath.py'
=== target is '../../_pythonpath.py'
=== added file 'setup.cfg'
--- setup.cfg	1970-01-01 00:00:00 +0000
+++ setup.cfg	2009-05-29 20:30:34 +0000
@@ -0,0 +1,14 @@
+[easy_install]
+# These settings control the behavior of the bootstrap.py script.  They keep
+# the version of setuptools and zc.buildout found in the download-cache as the
+# ones used to make the bin/buildout script.  Therefore, if your system Python
+# already has setuptools installed, this file will keep the system from going
+# out on the network for any distributions during the zc.buildout
+# bootstrapping.  Also, therefore, if you want to use a newer version of
+# setuptools or zc.buildout, you need to put it in ./download-cache/dist and
+# commit there, and change the specification in versions.cfg, as described
+# generally for dependencies in ./doc/buildout.txt.
+find_links = ./eggs
+             ./download-cache/dist
+allow_hosts = None
+

=== added file 'setup.py'
--- setup.py	1970-01-01 00:00:00 +0000
+++ setup.py	2009-05-29 20:30:34 +0000
@@ -0,0 +1,53 @@
+#!/usr/bin/env python
+
+# Copyright 2009 Canonical Ltd.  All rights reserved.
+
+import ez_setup
+ez_setup.use_setuptools()
+
+import sys
+from setuptools import setup, find_packages
+
+__version__ = '2.2.3'
+
+setup(
+    name='lp',
+    version=__version__,
+    packages=find_packages('lib'),
+    package_dir={'': 'lib'},
+    include_package_data=True,
+    zip_safe=False,
+    maintainer='Launchpad Developers',
+    description=('A unique collaboration and Bazaar code hosting platform '
+                 'for software projects.'),
+    license='LGPL v3',
+    install_requires=[
+        'feedvalidator',
+        'launchpadlib',
+        'lazr.uri',
+        'oauth',
+        'python-openid',
+        'setuptools',
+        'wadllib',
+    ],
+    url='https://launchpad.net/',
+    classifiers=[
+        "Development Status :: 5 - Production/Stable",
+        "Intended Audience :: Developers",
+        "Programming Language :: Python",
+    ],
+    extras_require=dict(
+        docs=[
+            'Sphinx',
+            'z3c.recipe.sphinxdoc',
+        ]
+    ),
+    entry_points=dict(
+        console_scripts=[ # `console_scripts` is a magic name to setuptools
+            'apiindex = lp.scripts.utilities.apiindex:main',
+            'killservice = lp.scripts.utilities.killservice:main',
+            'run = canonical.launchpad.scripts.runlaunchpad:start_launchpad',
+            'harness = canonical.database.harness:python',
+        ]
+    ),
+)

=== modified file 'test_on_merge.py'
--- test_on_merge.py	2008-06-02 03:17:01 +0000
+++ test_on_merge.py	2009-05-01 00:25:52 +0000
@@ -21,7 +21,7 @@
 TIMEOUT = 60 * 15
 
 def main():
-    """Call test.py with whatever arguments this script was run with.
+    """Call bin/test with whatever arguments this script was run with.
 
     If the tests ran ok (last line of stderr is 'OK<return>') then suppress
     output and exit(0).
@@ -134,7 +134,7 @@
 
     print 'Running tests.'
     os.chdir(here)
-    cmd = [sys.executable, 'test.py'] + sys.argv[1:]
+    cmd = [os.path.join(here, 'bin', 'test')] + sys.argv[1:]
     print ' '.join(cmd)
 
     # Run the test suite and return the error code
@@ -181,7 +181,7 @@
 def killem(pid, signal):
     """Kill the process group leader identified by pid and other group members
 
-    Note that test.py sets its process to a process group leader.
+    Note that bin/test sets its process to a process group leader.
     """
     try:
         os.killpg(os.getpgid(pid), signal)

=== added symlink 'utilities/_pythonpath.py'
=== target is '../_pythonpath.py'
=== removed file 'utilities/_pythonpath.py'
--- utilities/_pythonpath.py	2008-08-11 15:42:37 +0000
+++ utilities/_pythonpath.py	1970-01-01 00:00:00 +0000
@@ -1,12 +0,0 @@
-# Copyright 2004-2005 Canonical Ltd.  All rights reserved.
-
-__metaclass__ = type
-
-import sys, os, os.path
-
-sys.path.insert(0, os.path.join(
-    os.path.dirname(__file__), os.pardir, 'lib'
-    ))
-sys.path.insert(0, os.path.join(
-    os.path.dirname(__file__), os.pardir, 'lib', 'mailman'
-    ))

=== modified file 'utilities/link-external-sourcecode'
--- utilities/link-external-sourcecode	2008-10-09 10:31:30 +0000
+++ utilities/link-external-sourcecode	2009-05-14 20:46:25 +0000
@@ -36,14 +36,31 @@
 def gen_missing_files(source, destination):
     """Generate info on every file in source not in destination.
 
-    Yields `(source, destination, broken)` tuples, where `broken`
-    means that the destination is a broken symbolic link.
+    Yields `(source, destination)` tuples.
     """
     for name in listdir(source):
         destination_file = join(destination, name)
         if not exists(destination_file):
             source_file = join(source, name)
-            yield source_file, destination_file, islink(destination_file)
+            yield source_file, destination_file,
+
+
+def link(source, destination):
+    """Symlink source to destination.
+
+    Assumes destination is missing or broken.
+    """
+    try:
+        if islink(destination):
+            unlink(destination)
+        symlink(source, destination)
+    except OSError, error:
+        stderr.write(
+            '  Error linking %s: %s\n' % (basename(destination), error))
+    else:
+        if options.verbose:
+            stdout.write('%s -> %s\n' % (
+                    sep.join(destination.rsplit(sep, 3)[-3:]), source))
 
 
 if __name__ == '__main__':
@@ -98,18 +115,13 @@
         realpath(join(options.parent, 'sourcecode')),
         abspath(join(options.target, 'sourcecode')))
 
-    for target, link, broken in missing_files:
-        try:
-            if broken:
-                unlink(link)
-            symlink(target, link)
-        except OSError, error:
-            stderr.write(
-                '  Error linking %s: %s\n' % (basename(link), error))
-        else:
-            if options.verbose:
-                stdout.write('%s -> %s\n' % (
-                        sep.join(link.rsplit(sep, 3)[-3:]), target))
+    for source, destination in missing_files:
+        link(source, destination)
 
+    for folder_name in ('eggs', 'download-cache'):
+        source = join(options.parent, folder_name)
+        destination = join(options.target, folder_name)
+        if not exists(destination):
+            link(source, destination)
 
 # End

=== modified file 'utilities/rocketfuel-branch'
--- utilities/rocketfuel-branch	2008-07-11 14:10:35 +0000
+++ utilities/rocketfuel-branch	2009-05-06 21:06:22 +0000
@@ -9,6 +9,11 @@
     exit 1
 fi
 
+LP_DOWNLOAD_CACHE_PATH=$LP_PROJECT_ROOT/$LP_SOURCEDEPS_DIR/buildout/download-cache
+LP_EGGS_PATH=$LP_PROJECT_ROOT/$LP_SOURCEDEPS_DIR/buildout/eggs
+LP_DOWNLOAD_CACHE_PATH=$(eval echo ${LP_DOWNLOAD_CACHE_PATH})
+LP_EGGS_PATH=$(eval echo ${LP_EGGS_PATH})
+
 if [ "x$1" == "x" ]; then
     echo "Usage: $0 new-branch-name"
     echo "Example: '$0 fixes-bug-54356'"

=== removed file 'utilities/rocketfuel-env.sh'
--- utilities/rocketfuel-env.sh	2008-06-30 21:37:42 +0000
+++ utilities/rocketfuel-env.sh	1970-01-01 00:00:00 +0000
@@ -1,28 +0,0 @@
-# Common environment variables for the rocketfuel-* scripts.
-#
-# The ones you can set are:
-#
-# LP_PROJECT_ROOT - The root directory of all your Canonical stuff.  Your
-#                   Launchpad shared repository will live in a child directory
-#                   of this directory.
-# LP_SHARED_REPO  - Your Launchpad shared repository directory.  All of your
-#                   Launchpad development branches will live under here.
-# LP_TRUNK_NAME   - The directory name (not path!) to your rocketfuel trunk
-#                   mirror directory.  This is relative to your shared repo.
-# LP_SOURCEDEPS_DIR - The name of the directory (not path!) where your
-#                   trunk sourcecode will be placed.  This is relative to your
-#                   LP_PROJECT_ROOT and should /not/ have the 'sourcecode'
-#                   path appended to it, since this is automatically added by
-#                   the scripts.
-
-LP_PROJECT_ROOT=${LP_PROJECT_ROOT:=~/canonical}
-LP_SHARED_REPO=${LP_SHARED_REPO:=lp-branches}
-LP_PROJECT_PATH=$LP_PROJECT_ROOT/$LP_SHARED_REPO
-LP_TRUNK_NAME=${LP_TRUNK_NAME:=trunk}
-LP_TRUNK_PATH=$LP_PROJECT_PATH/$LP_TRUNK_NAME
-
-LP_SOURCEDEPS_DIR=${LP_SOURCEDEPS_DIR:=lp-sourcedeps}
-LP_SOURCEDEPS_PATH=$LP_PROJECT_ROOT/$LP_SOURCEDEPS_DIR/sourcecode
-
-# Force tilde expansion
-LP_SOURCEDEPS_PATH=$(eval echo ${LP_SOURCEDEPS_PATH})

=== modified file 'utilities/rocketfuel-get'
--- utilities/rocketfuel-get	2008-07-11 14:10:35 +0000
+++ utilities/rocketfuel-get	2009-05-14 20:46:25 +0000
@@ -10,6 +10,11 @@
     exit 1
 fi
 
+LP_DOWNLOAD_CACHE_PATH=$LP_PROJECT_ROOT/$LP_SOURCEDEPS_DIR/download-cache
+LP_EGGS_PATH=$LP_PROJECT_ROOT/$LP_SOURCEDEPS_DIR/eggs
+LP_DOWNLOAD_CACHE_PATH=$(eval echo ${LP_DOWNLOAD_CACHE_PATH})
+LP_EGGS_PATH=$(eval echo ${LP_EGGS_PATH})
+
 DEVPAD="devpad.canonical.com"
 ROCKETFUELDEPS="/code/rocketfuel-built/launchpad/sourcecode/*"
 RSYNCOPTIONS="-avp --partial --progress --delete"
@@ -25,8 +30,13 @@
 bzr pull
 FINAL_REV=$(bzr revno)
 
-# Update the source dependencies
+# Make sure our directories are around
 if [ ! -d $LP_SOURCEDEPS_PATH ]; then mkdir -p $LP_SOURCEDEPS_PATH; fi
+if [ ! -d $LP_EGGS_PATH ]; then mkdir -p $LP_EGGS_PATH; fi
+
+# Get/update the download cache
+if [ ! -d $LP_DOWNLOAD_CACHE_PATH ]; then bzr co lp:lp-source-dependencies $LP_DOWNLOAD_CACHE_PATH; fi
+bzr up $LP_DOWNLOAD_CACHE_PATH
 
 # Several (*) have tried, and failed, to get the --filter to fit neatly
 # into RSYNCOPTIONS, so we put it here. Bash arrays do not count.

=== modified file 'utilities/rocketfuel-setup'
--- utilities/rocketfuel-setup	2009-05-13 05:43:51 +0000
+++ utilities/rocketfuel-setup	2009-05-29 00:39:22 +0000
@@ -471,12 +471,6 @@
   echo "Setup ssh key for local supermirror ssh access."
 fi
 
-# Setup environment variables in a ~ dot file
-cd $HOME
-if [ ! -e $HOME/.rocketfuel-env.sh ]; then
-  ln -s $LP_TRUNK_PATH/utilities/rocketfuel-env.sh .rocketfuel-env.sh
-fi
-
 # Setup scripts in /usr/local/bin
 cd /usr/local/bin
 if [ ! -e rocketfuel-get ]; then
@@ -550,13 +544,14 @@
     branches you are working on.
 
  rocketfuel-branch foo
-    Create a new branch of LP called "foo" in $LP_PROJECT_PATH/foo,
+    Create a new branch of LP called \"foo\" in
+    $LP_PROJECT_PATH/foo,
     with all the source dependencies properly linked in.
 
  rocketfuel-status
-    Check each of the branches in $LP_PROJECT_PATH and show which of them have
-    uncommitted changes, also check which ones have revisions that have not
-    yet landed on trunk.
+    Check each of the branches in $LP_PROJECT_PATH
+    and show which of them have uncommitted changes, also check which ones
+    have revisions that have not yet landed on trunk.
 
  rocketfuel-push
     Push all of your branches to Launchpad, so that you have a server-side

=== added file 'versions.cfg'
--- versions.cfg	1970-01-01 00:00:00 +0000
+++ versions.cfg	2009-05-31 11:46:25 +0000
@@ -0,0 +1,28 @@
+[buildout]
+versions = versions
+
+[versions]
+feedvalidator = 0.0.0DEV-r1049
+httplib2 = 0.4.0
+ipython = 0.9.1
+launchpadlib = 1.0.1
+lazr.uri = 1.0
+oauth = 1.0
+python-openid = 2.2.1
+setuptools = 0.6c9
+simplejson = 1.7.5
+wadllib = 0.1
+z3c.recipe.filetemplate = 2.0.2
+z3c.recipe.i18n = 0.5.3
+z3c.recipe.tag = 0.2.0
+zc.buildout = 1.2.1
+zc.lockfile = 1.0.0
+zc.recipe.egg = 1.2.2
+ZConfig = 2.6.1
+zope.app.locales = 3.5.1
+zope.configuration = 3.6.0
+zope.event = 3.4.1
+zope.i18nmessageid = 3.4.3
+zope.interface = 3.5.1
+zope.schema = 3.5.4
+zope.testing = 3.7.4

